// -*- mode: C++; c-file-style: "stroustrup"; c-basic-offset: 4; indent-tabs-mode: nil; -*-

#ifndef TRON_SOCKETS_H
#define TRON_SOCKETS_H

#include "tron/comm.h"

/*
Exceptions are not portable. Not to Windows (see pthreadGCE.dll).
class TronException {
public:
    const char *method, *message;
    int number;
    TronException(const char* _method);
    TronException(const char* _method, int errornumber);
    std::ostream& print(std::ostream&);
// (feel free to write your own << operators)
};
*/

/**
 * TCP/IP socket wrapper for writing and reading primitive data
 */
class ClientSocket: public DuplexConnection
{
    int sock;
    const char* host;
    int port;

public:
// create a socket on a descriptor (used by ServerSocket)
    ClientSocket(int descriptor, size_t maxWriteBufferSize=256):
        DuplexConnection(maxWriteBufferSize), sock(descriptor), 
        host(0), port(0)
        {}
// connect to remote TCP/IP socket, use connect() afterwards:
    ClientSocket(const char* _host, int _port, size_t maxWriteBufferSize=256):
        DuplexConnection(maxWriteBufferSize), sock(-1), host(_host), port(_port)
        {}
    ~ClientSocket();
    int32_t connect(int retries, int delay_in_secs);
    void closeConnection();
    bool connected() { return (sock>=0); }
    int32_t setLowLatency(); // sets the TCP_NODELAY option
    int32_t readByte(char& c);
    int32_t readInt16(int16_t&);
    int32_t readInt32(int32_t&);
    int32_t readInt64(int64_t& i);
    int32_t readNString(char* buffer);

    int32_t writeBuffer(const char* buffer, size_t len);
};

/**
 * Local TCP/IP socket for accepting connections
 */
class ServerSocket
{
    int sock;
public:
    ServerSocket(int port, int connection_buffer_size=5);
    ~ServerSocket();
    ClientSocket* accept();
};

#endif
